<?php
// +-------------------------------------------------+
// | 2002-2011 PMB Services / www.sigb.net pmb@sigb.net et contributeurs (voir www.sigb.net)
// +-------------------------------------------------+
// $Id: list_readers_recouvr_ui.class.php,v 1.1.2.5 2020/07/13 06:22:23 dgoron Exp $

if (stristr($_SERVER['REQUEST_URI'], ".class.php")) die("no access");

global $class_path;
require_once($class_path."/list/readers/list_readers_ui.class.php");
require_once($class_path."/comptes.class.php");

class list_readers_recouvr_ui extends list_readers_ui {
	
	protected function _get_query_base() {
	    $query = "SELECT empr.*, round(sum(id_expl!=0)/2) as nb_ouvrages, sum(montant) as somme ,location_libelle 
			FROM recouvrements
			JOIN empr ON id_empr=empr_id
			JOIN docs_location ON empr_location=idlocation";
	    return $query;
	}
	
	protected function _get_query_order() {
		return ' GROUP BY empr.id_empr '.parent::_get_query_order();
	}
	
	protected function add_object($row) {
	    $this->objects[] = $row;
	}
	
	/**
	 * Initialisation des filtres de recherche
	 */
	public function init_filters($filters=array()) {
		parent::init_filters($filters);
		$this->filters['empr_location_id'] = '';
	}
	
	/**
	 * Affichage du formulaire de recherche
	 */
	public function get_search_form() {
		$search_form = $this->get_search_hidden_form();
		$search_form = str_replace('!!action!!', static::get_controller_url_base(), $search_form);
		return $search_form;
	}
	
	protected function _get_query_human() {
		return '';
	}
	
	/**
	 * Initialisation des colonnes disponibles
	 */
	protected function init_available_columns() {
		parent::init_available_columns();
		$this->available_columns['main_fields']['nb_ouvrages'] = 'relance_recouvrement_nb_ouvrages';
		$this->available_columns['main_fields']['somme_totale'] = 'relance_recouvrement_somme_totale';
	}
	
	protected function init_default_columns() {
		global $pmb_lecteurs_localises;
		
		$this->add_column('cb', 'relance_recouvrement_cb');
		$this->add_column('empr_name', 'relance_recouvrement_name');
		if($pmb_lecteurs_localises) {
			$this->add_column('location', 'empr_location');
		}
		$this->add_column('nb_ouvrages');
		$this->add_column('somme_totale');
	}
	
	protected function init_default_settings() {
		parent::init_default_settings();
		$this->set_setting_column('default', 'align', 'left');
		$this->set_setting_column('nb_ouvrages', 'align', 'center');
		$this->set_setting_column('somme_totale', 'align', 'right');
	}
	
	/**
	 * Affichage d'une colonne
	 * @param object $object
	 * @param string $property
	 */
	protected function get_display_cell($object, $property) {
		$attributes = array(
				'onmousedown' => "document.location=\"".$this->get_controller_url_base()."&act=recouvr_reader&id_empr=".$object->id_empr."\";"
		);
		$content = $this->get_cell_content($object, $property);
		$display = $this->get_display_format_cell($content, $property, $attributes);
		return $display;
	}
	
	protected function get_cell_content($object, $property) {
		$content = '';
		switch($property) {
			case 'cb':
				$content .= $object->empr_cb;
				break;
			case 'empr_name':
				$content .= $object->empr_nom." ".$object->empr_prenom;
				break;
			case 'location':
				$content .= $object->location_libelle;
				break;
			case 'somme_totale':
				$content .= comptes::format_simple($object->somme);
				break;
			default:
				$content .= parent::get_cell_content($object, $property);
				break;
		}
		return $content;
	}
	
	protected function get_selection_actions() {
		if(!isset($this->selection_actions)) {
			$this->selection_actions = array();
		}
		return $this->selection_actions;
	}
	
	public function get_export_icons() {
		return '';
	}
	
	public static function get_controller_url_base() {
		global $base_path;
		global $categ, $sub;
	
		return $base_path.'/circ.php?categ='.$categ.'&sub='.$sub;
	}
}